@@ -10,9 +10,9 @@ from optparse import make_option, OptionParser
 from setproctitle import setproctitle
 
 from limpyd import __version__ as limpyd_version
-from limpyd.exceptions import ImplementationError, DoesNotExist
+from limpyd.exceptions import DoesNotExist
 
-from limpyd_jobs import STATUSES
+from limpyd_jobs import STATUSES, LimpydJobsException, ConfigurationException
 from limpyd_jobs.models import Queue, Job, Error
 
 LOGGER_BASE_NAME = 'limpyd-jobs'
@@ -39,8 +39,12 @@ class Worker(object):
     # max delay for blpop
     timeout = 30
     # minimum time in seconds between the update of keys to fetch
-    fetch_priorities_delay = 25  # by default < timeout to be sure to
-                                 # fetch them after a timeout occured
+    # by default < timeout to be sure to fetch them after a timeout occured
+    fetch_priorities_delay = 25
+    # number of time to requeue a job after a failure
+    requeue_times = 0
+    # delta to add to the current priority of a job to be requeued
+    requeue_priority_delta = -1
 
     # we want to intercept SIGTERM and SIGINT signals, to stop gracefuly
     terminate_gracefuly = True
@@ -49,7 +53,8 @@ class Worker(object):
                  queue_model=None, job_model=None, error_model=None,
                  logger_base_name=None, logger_level=None, save_errors=None,
                  save_tracebacks=None, max_loops=None, terminate_gracefuly=None,
-                 timeout=None, fetch_priorities_delay=None):
+                 timeout=None, fetch_priorities_delay=None, requeue_times=None,
+                 requeue_priority_delta=None):
         """
         Create the worker by saving arguments, doing some checks, preparing
         logger and signals management, and getting queues keys.
@@ -58,7 +63,7 @@ class Worker(object):
             self.name = name
 
         if not self.name:
-            raise ImplementationError('The name of the worker is not defined')
+            raise ConfigurationException('The name of the worker is not defined')
 
         # save and check models to use
         if queue_model is not None:
@@ -85,6 +90,10 @@ class Worker(object):
             self.timeout = timeout
         if fetch_priorities_delay is not None:
             self.fetch_priorities_delay = fetch_priorities_delay
+        if requeue_times is not None:
+            self.requeue_times = requeue_times
+        if requeue_priority_delta is not None:
+            self.requeue_priority_delta = requeue_priority_delta
 
         # prepare logging
         if logger_base_name is not None:
@@ -113,8 +122,8 @@ class Worker(object):
         to describe the name of the argument.
         """
         if not issubclass(model_to_check, model_reference):
-            raise ImplementationError('The %s model must be a subclass of %s' % (
-                                      obj_name, model_reference.__name__))
+            raise ConfigurationException('The %s model must be a subclass of %s'
+                                         % (obj_name, model_reference.__name__))
 
     def handle_end_signal(self):
         """
@@ -278,7 +287,7 @@ class Worker(object):
         # if status is not None, we already had a run !
         if self.status:
             self.set_status('aborted')
-            raise ImplementationError('This worker run is already terminated')
+            raise LimpydJobsException('This worker run is already terminated')
 
         self.set_status('starting')
 
@@ -372,12 +381,21 @@ class Worker(object):
         if self.save_errors:
             additional_fields = self.additional_error_fields(job, queue, exception)
             self.error_model.add_error(queue_name=queue._cached_name,
-                                       identifier=job._cached_identifier,
+                                       job=job,
                                        error=exception,
                                        trace=trace,
                                        **additional_fields)
+
         self.log(self.job_error_message(job, queue, exception, trace), level='error')
 
+        if self.requeue_times and self.requeue_times >= int(job.tries.hget() or 0):
+            name, priority = queue.hmget('name', 'priority')
+            if self.requeue_priority_delta:
+                priority = int(priority) + self.requeue_priority_delta
+            job.requeue(name, priority, self.queue_model)
+            self.log('[%s|%s] requeued' % (queue._cached_name,
+                                                        job._cached_identifier))
+
     def job_error_message(self, job, queue, exception, trace=None):
         """
         Return the message to log when a job raised an error
@@ -406,6 +424,7 @@ class Worker(object):
         Called just before the execution of the job
         """
         job.hmset(start=str(datetime.utcnow()), status=STATUSES.RUNNING)
+        job.tries.hincrby(1)
         self.log(self.job_started_message(job, queue))
 
     def job_started_message(self, job, queue):
@@ -500,6 +519,12 @@ class WorkerConfig(object):
         make_option('--fetch-priorities-delay', type='int', dest='fetch_priorities_delay',
             help='Min delay (seconds) to wait before fetching new priority queues, e.g. --fetch-priorities-delay=30'),
 
+        make_option('--requeue-times', type='int', dest='requeue_times',
+            help='Number of time to requeue a failing job (default to 0), e.g. --requeue-times=5'),
+
+        make_option('--requeue-priority-delta', type='int', dest='requeue_priority_delta',
+            help='Delta to add to the actual priority of a failing job to be requeued (default to -1, ie one level lower), e.g. --requeue-priority-delta=-2'),
+
         make_option('--database', action='store', dest='database',
             help='Redis database to use (host:port:db), e.g. --database=localhost:6379:15'),
 
@@ -518,7 +543,8 @@ class WorkerConfig(object):
     worker_options = ('name', 'job_model', 'queue_model', 'error_model',
                       'callback', 'logger_base_name', 'logger_level',
                       'save_errors', 'save_tracebacks', 'max_loops',
-                      'terminate_gracefuly', 'timeout', 'fetch_priorities_delay')
+                      'terminate_gracefuly', 'timeout', 'fetch_priorities_delay',
+                      'requeue_times', 'requeue_priority_delta')
 
     @staticmethod
     def _import_module(module_uri):
@@ -619,7 +645,10 @@ class WorkerConfig(object):
             self.parser.error('The timeout argument (%s) must be a positive integer (including 0)' % self.options.timeout)
 
         if self.options.fetch_priorities_delay is not None and self.options.fetch_priorities_delay <= 0:
-            self.parser.error('The fetch_priorities_delay argument (%s) must be a positive integer' % self.options.fetch_priorities_delay)
+            self.parser.error('The fetch-priorities-delay argument (%s) must be a positive integer' % self.options.fetch_priorities_delay)
+
+        if self.options.requeue_times is not None and self.options.requeue_times < 0:
+            self.parser.error('The requeue-times argument (%s) must be a positive integer (including 0)' % self.options.requeue_times)
 
         self.database_config = None
         if self.options.database:
